@@ -41,6 +41,7 @@
 #include "dealer.h"
 #include "dealerinfo.h"
 #include "gameselectionscene.h"
+#include "kpat_debug.h"
 #include "numbereddealdialog.h"
 #include "renderer.h"
 #include "settings.h"
@@ -60,7 +61,6 @@
 #include <QAction>
 #include <KActionCollection>
 #include <KConfigDialog>
-#include <QDebug>
 #include <QFileDialog>
 #include <QIcon>
 #include <KLocalizedString>
@@ -78,6 +78,7 @@
 #include <QList>
 #include <QPointer>
 #include <QTimer>
+#include <QBuffer>
 #include <QXmlStreamReader>
 #include <QDesktopWidget>
 #include <QKeySequence>
@@ -284,7 +285,7 @@ void MainWindow::setupActions()
 
 
     // Hidden actions
-    if (!qgetenv("KDE_DEBUG").isEmpty()) // developer shortcut
+    if (!qEnvironmentVariableIsEmpty("KDE_DEBUG")) // developer shortcut
     {
         a = actionCollection()->addAction( QStringLiteral( "themePreview" ));
         a->setText(i18n("Generate a theme preview image"));
