@@ -48,6 +48,12 @@ SolverInterface::ExitStatus FcSolveSolver::patsolve( int _max_positions )
 
     init();
 
+    int no_use = 0;
+    int num_moves = 0;
+    get_possible_moves(&no_use, &num_moves);
+    Q_ASSERT( m_firstMoves.count() == 0 );
+    for (int j = 0; j < num_moves; ++j)
+        m_firstMoves.append( Possible[j] );
     if (!solver_instance)
     {
         {
@@ -146,6 +152,7 @@ SolverInterface::ExitStatus FcSolveSolver::patsolve( int _max_positions )
 
                         Q_ASSERT (verdict);
 
+                        new_move.is_fcs = true;
                         new_move.fcs = move;
 
                         m_winMoves.append( new_move );
@@ -172,11 +179,6 @@ SolverInterface::ExitStatus FcSolveSolver::patsolve( int _max_positions )
 
 /* Get the possible moves from a position, and store them in Possible[]. */
 
-int FcSolveSolver::get_possible_moves(int *, int *)
-{
-    return 0;
-}
-
 bool FcSolveSolver::isWon()
 {
     return true;
